/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSPersistentDocument;

import dstep.appkit.NSDocument;
import dstep.coredata.NSManagedObjectContext;
import dstep.coredata.NSManagedObjectModel;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSError;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

const TNSPersistentDocumentDeprecated = `

	bool configurePersistentStoreCoordinatorForURL (NSURL url, NSString fileType, NSError** error)
	{
		return invokeObjcSelf!(bool, "configurePersistentStoreCoordinatorForURL:ofType:error:", NSURL, NSString, NSError**)(url, fileType, error);
	}

	//mixin ObjcBindMethod!(configurePersistentStoreCoordinatorForURL, "configurePersistentStoreCoordinatorForURL:ofType:error:");

`;

class NSPersistentDocument : NSDocument
{
	mixin (ObjcWrap);

	NSManagedObjectContext managedObjectContext ()
	{
		return invokeObjcSelf!(NSManagedObjectContext, "managedObjectContext");
	}

	void setManagedObjectContext (NSManagedObjectContext managedObjectContext)
	{
		return invokeObjcSelf!(void, "setManagedObjectContext:", NSManagedObjectContext)(managedObjectContext);
	}

	Object managedObjectModel ()
	{
		return invokeObjcSelf!(Object, "managedObjectModel");
	}

	bool configurePersistentStoreCoordinatorForURL (NSURL url, NSString fileType, NSString configuration, NSDictionary storeOptions, NSError** error)
	{
		return invokeObjcSelf!(bool, "configurePersistentStoreCoordinatorForURL:ofType:modelConfiguration:storeOptions:error:", NSURL, NSString, NSString, NSDictionary, NSError**)(url, fileType, configuration, storeOptions, error);
	}

	NSString persistentStoreTypeForFileType (NSString fileType)
	{
		return invokeObjcSelf!(NSString, "persistentStoreTypeForFileType:", NSString)(fileType);
	}

	bool writeToURL (NSURL absoluteURL, NSString typeName, uint saveOperation, NSURL absoluteOriginalContentsURL, ref NSError error)
	{
		id err;
		
		if (error)
			err = error.objcObject;
		
		bool result = invokeObjcSelf!(bool, "writeToURL:ofType:forSaveOperation:originalContentsURL:error:", NSURL, NSString, uint, NSURL, id*)(absoluteURL, typeName, saveOperation, absoluteOriginalContentsURL, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	bool readFromURL (NSURL absoluteURL, NSString typeName, ref NSError error)
	{
		id err;
		
		if (error)
			err = error.objcObject;
		
		bool result = invokeObjcSelf!(bool, "readFromURL:ofType:error:", NSURL, NSString, id*)(absoluteURL, typeName, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	bool revertToContentsOfURL (NSURL inAbsoluteURL, NSString inTypeName, ref NSError outError)
	{
		id error;
		
		if (outError)
			error = outError.objcObject;
		
		bool result = invokeObjcSelf!(bool, "revertToContentsOfURL:ofType:error:", NSURL, NSString, id*)(inAbsoluteURL, inTypeName, &error);
		
		if (error)
			outError = new NSError(error);
		
		return result;
	}
	
	// NSPersistentDocumentDeprecated
	bool configurePersistentStoreCoordinatorForURL (NSURL url, NSString fileType, NSError** error)
	{
		return invokeObjcSelf!(bool, "configurePersistentStoreCoordinatorForURL:ofType:error:", NSURL, NSString, NSError**)(url, fileType, error);
	}
}